/////////////////////////////////////////////////////////////////////////////
// Include files
// These lines allow us to use code routines (libraries) from other files,
// so that we don't have to write everything by ourselves.
/////////////////////////////////////////////////////////////////////////////

#include <p18lf25k22.h>	    // This file includes definitions for all of the
                            // registers on our chip
#include <math.h>

/////////////////////////////////////////////////////////////////////////////
// Pragma statements
// These lines tell the microcontroller what configuration to use when it
// when it turns on. The most important part for now is to tell the
// microcontroller what to use for a clock input.
/////////////////////////////////////////////////////////////////////////////

// Use the internal oscillator as a clock
#pragma config FOSC = INTIO67

// Configure it to run at 16 MHz
#pragma config PLLCFG = OFF

// Make sure that the internal watchdog timer is off. The watchdog is a
// a special feature of the processor, that runs separately from the main
// program 
#pragma config WDTEN = OFF
#pragma config PBADEN = OFF

#define numSteps 400

/////////////////////////////////////////////////////////////////////////////
// Function declarations
// Declare any user functions that you want to use here
/////////////////////////////////////////////////////////////////////////////
void main (void);
void setup(void);
void loop(void);

// Bit masks for the motor state tables
int A_C1states[4] = {0x00, 0x01, 0x01, 0x00};
int A_C2states[4] = {0x02, 0x02, 0x00, 0x00};
int B_C1states[4] = {0x00, 0x04, 0x04, 0x00};
int B_C2states[4] = {0x08, 0x08, 0x00, 0x00};

int Astate = 0;       // state of left motor
int Bstate = 0;       // state of right motor
int stepUnit = 142;   // calculated from spool diameter
int w;                // width of drawing field
int h;                // height
int a1;               // length of left string
int b1;               // length of right string
int x1;               // current x position
int y1;               // current y position

// Function definitions
// Define any user functions that you want to use here
/////////////////////////////////////////////////////////////////////////////

// The main() function is where the program 'starts' when the microcontroller
// is turned on. For this project, we will use it to call setup() and loop()
// functions, then use those similar to an Arduino sketch.

void main (void)
{
    // Call the setup function once to put everything in order
    setup();

    // Then call the loop() function over and over
    while (1)
    {
        loop();
    }
}

void turnMotors(int stepDirA, int stepDirB) {

    // turn the left and right motors one step in the given direction

    unsigned int count;
    a1 += stepDirA;
    b1 += stepDirB;
    Astate-=stepDirA;
    Bstate+=stepDirB;
    if (Astate == -1) { Astate = 3; }
    if (Bstate == -1) { Bstate = 3; }
    Astate = Astate %4;
    Bstate = Bstate %4;

    PORTA = A_C1states[Astate] | A_C2states[Astate] | 
            B_C1states[Bstate] | B_C2states[Bstate] ;

    for( count = 0; count < 100; count++) {}   // delay
}

void moveTo(int x2, int y2) {

    // move from the current point (x1, y1) to (x2, y2)
    // implement Breshenham's algorithm to straighten this out

    int dirA =0;
    int dirB=0;
    int a2 = sqrt(pow(x2, 2)+pow(y2, 2));
    int b2 = sqrt(pow((w-x2), 2)+pow(y2,2));
      if (a2 > a1) { dirA = 1; }
      if (a2 < a1) { dirA = -1; }
      if (b2 > b1) { dirB = 1; }
      if (b2 < b1) { dirB = -1; }
    while ((a1!=a2) || (b1!=b2)) {
      if (a1 == a2) { dirA = 0; }
      if (b1 == b2) { dirB = 0; }
      turnMotors(dirA, dirB);
    }
    x1=x2;
    y1=y2;
}

// This function is called once, when the microcontroller is turned on.

void setup(void) {
    // Set all of the pins on Port A to be outputs,
    // Cleared bits are outputs.
    TRISA = 0x00;
    PORTA = 0x00;

    // calculate width of drawing area in steps

    w = 72*stepUnit;
    h = 36*stepUnit;

    // set home position
    x1 = w/2;
    y1 = h;

    // calculate starting lengths

    a1 = sqrt(pow(x1, 2)+pow(y1, 2));
    b1 = sqrt(pow((w-x1), 2)+pow(y1,2));
}

// This function is called repeatedly

void loop(void) {

   // draw in a square

   moveTo(w/2-400, h);
   moveTo(w/2-400, h-800);
    moveTo(w/2+400, h-800);
    moveTo(w/2+400, h);
    moveTo(w/2, h);
}
